@@ -46,13 +46,36 @@
 
 #include <config.h>
 #include <string.h>
+#include <stdlib.h>
 #include <gtk/gtk.h>
 #include <glade/glade.h>
+#include <glade/glade-build.h>
 
-#ifdef WITH_BONOBOUI
-#  include <libbonoboui.h>
+#ifdef WITH_GNOME
+#  include <libgnomeui/libgnomeui.h>
 #endif
 
+static void
+run_tests (void)
+{
+    g_print ("Testing enum to string\n");
+    g_assert (glade_enum_from_string (GTK_TYPE_ANCHOR_TYPE,
+				      "GTK_ANCHOR_NORTH_WEST") ==
+	      GTK_ANCHOR_NORTH_WEST);
+
+    g_print ("Testing flags to string\n");
+    g_assert (glade_flags_from_string (GTK_TYPE_ATTACH_OPTIONS,
+				       "expand|fill") ==
+	      (GTK_EXPAND | GTK_FILL));
+
+    g_assert (glade_flags_from_string (GTK_TYPE_ATTACH_OPTIONS,
+				       "fill|expand") ==
+	      (GTK_EXPAND | GTK_FILL));
+
+    g_print ("All tests passed\n");
+    exit (0);
+}
+
 int
 main (int argc, char **argv)
 {
@@ -65,18 +88,24 @@ main (int argc, char **argv)
 
     /* initialise libraries */
     gtk_init(&argc, &argv);
-#ifdef WITH_BONOBOUI
-    if (!bonobo_ui_init ("test-libglade", VERSION, &argc, argv))
-	g_error (_("Can not bonobo_ui_init"));
+
+#ifdef WITH_GNOME
+    if (!gnome_program_init ("Terminal", VERSION,
+			     LIBGNOMEUI_MODULE,
+			     argc, argv,
+			     NULL))
+	g_error ("Cannot gnome_program_init ()");
 #endif
 
-    /* initialise libglade itself */
-    glade_init();
+    if (g_getenv ("LIBGLADE_REGRESSION"))
+	    run_tests ();
 
     /* argument parsing */
     for (i = 1; i < argc; i++) {
 	if (!strcmp(argv[i], "--no-connect"))
 	    no_connect = TRUE;
+	else if (!strcmp(argv[i], "--run-tests"))
+	    run_tests ();
 	else if (filename == NULL)
 	    filename = argv[i];
 	else if (rootnode == NULL)
@@ -96,6 +125,19 @@ main (int argc, char **argv)
     if (!xml) {
 	g_warning("something bad happened while creating the interface");
 	return 1;
+    } else {
+	/* show toplevels */
+	int    i;
+	char **tops;
+
+	tops = glade_xml_get_toplevel_names (xml);
+
+	for (i = 0; tops && tops [i]; i++) {
+	    GtkWidget *w = glade_xml_get_widget (xml, tops [i]);
+	    gtk_widget_show (w);
+	}
+
+	g_strfreev (tops);
     }
 
     /* if you pass the rootnode argument to libglade, it will only
